<?php
// +----------------------------------------------------------------------
// | VitPHP
// +----------------------------------------------------------------------
// | 版权所有 2018~2021 藁城区创新网络电子商务中心 [ http://www.vitphp.cn ]
// +----------------------------------------------------------------------
// | VitPHP是一款免费开源软件,您可以访问http://www.vitphp.cn/以获得更多细节。
// +----------------------------------------------------------------------

namespace app\app_kami\controller;

use app\BaseController;
use think\App;
use think\facade\Db;
use vitphp\admin\traits\Jump;

class Base extends BaseController
{
    use Jump;

    protected $res = [
        'code' => 0,
        'msg' => 'ok',
        'data' => ''
    ];

    /**
     * Base constructor.
     */
    public function __construct(App $app)
    {
        parent::__construct($app);

        $this->cannonFodder();
        $this->getWechatInfo();
        $this->removeCode();
    }

    /**
     * 获取当前活动的用户信息,没有就新建
     */
    protected function getUserInfo()
    {
        $gid = input('id');
        $pid = input('pid');
        $wechatInfo = session('wechat_info_' . $pid);
        if(!$gid) $this->error('活动不存在');

        $user = Db::name('app_kami_user')->where(['pid'=>$pid,'gid'=>$gid,'openid'=>$wechatInfo['openid']])->find();
        if($user) return $user;

        Db::name('app_kami_user')->insert([
            'pid'=>$pid,
            'gid'=>$gid,
            'openid'=>$wechatInfo['openid'],
            'nickname'=>$wechatInfo['nickname'],
            'headimgurl'=>$wechatInfo['headimgurl'],
            'create_time'=>time()
        ]);

        $user = Db::name('app_kami_user')->where(['pid'=>$pid,'gid'=>$gid,'openid'=>$wechatInfo['openid']])->find();
        return $user;
    }

    /**
     * 处理炮灰域名
     * 先判断是否配置了炮灰域名，若配置，再判断当前域名是否为炮灰域名
     * 配置了，且当前不是炮灰域名，就跳转到炮灰域名
     */
    protected function cannonFodder()
    {
        $cannonFodderRange = [];
        if ($cannonFodderRange && !in_array($this->request->host(), $cannonFodderRange)) {
            $paohuiDomain = '';
            $thisUrl = $this->request->scheme() . '://' . $paohuiDomain . $this->request->url();
            redirect($thisUrl)->send();
        }
    }

    /**
     * 剔除code与state
     */
    protected function removeCode()
    {
        if (!$this->request->isGet() || $this->request->isAjax()) return;
        if (!isset($_GET['code']) && !isset($_GET['state'])) return;

        $pid = input('pid');
        $wechatInfo = session('wechat_info_' . $pid);
        if (!$wechatInfo) return;
        $baseUrl = $this->request->domain() . $this->request->baseUrl();
        $url = $baseUrl;

        unset($_GET['code']);
        unset($_GET['state']);
        if ($_GET) {
            $url = $baseUrl . '?' . http_build_query($_GET);
        }
        \think\facade\Session::save();
        echo "<script>window.location.href='{$url}'</script>";
    }

    /**
     * 获取微信授权信息
     * @return array|mixed|Db|\think\Model|null
     * @throws \WeChat\Exceptions\InvalidResponseException
     * @throws \WeChat\Exceptions\LocalCacheException
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    protected function getWechatInfo()
    {
        if (!$this->request->isGet() || $this->request->isAjax()) return [];

        $pid = input('pid');
        $wechatInfo = session('wechat_info_' . $pid);
        $thisUrl = $this->request->domain() . $this->request->url();
        $weixinConfig = Db::name('app_kami_weixin')->where(['pid' => $pid])->find();
        $config = [
            'appsecret' => $weixinConfig['appsecret'],
            'appid' => $weixinConfig['appid'],
        ];
        if (!$wechatInfo || !$wechatInfo['openid']) {
            $wechat = new \WeChat\Oauth($config);
            if (!input('code')) {
                $state = '123';
                $tiaoUrl = $thisUrl;
                if ($weixinConfig['authdomain']) {
                    $tiaoUrl = $weixinConfig['authdomain'];
                    $stateKey = md5(session_id() . $pid . md5(time()) . $thisUrl . time());
                    $state = 'vitphp-' . $stateKey;
                    cache($state, $thisUrl, 60);
                }
                $redirectUrl = $wechat->getOauthRedirect($tiaoUrl, $state, 'snsapi_userinfo');
                redirect($redirectUrl)->send();
            } else {
                $result = $wechat->getOauthAccessToken();
                $wechatInfo = $wechat->getUserInfo($result['access_token'], $result['openid']);
                session('wechat_info_' . $pid, $wechatInfo);
                \think\facade\Session::save();
            }
        }
        return $wechatInfo;
    }

    protected function getWechatJsConfig()
    {
        $pid = input('pid');
        $thisUrl = $this->request->domain() . $this->request->url();
        $weixinConfig = Db::name('app_kami_weixin')->where(['pid' => $pid])->find();
        $config = [
            'appsecret' => $weixinConfig['appsecret'],
            'appid' => $weixinConfig['appid'],
        ];
        $scriptWechat = new \WeChat\Script($config);
        $wechatConfig = $scriptWechat->getJsSign($thisUrl);
        $jsApiList = '';
        foreach ($wechatConfig['jsApiList'] as $jsApi) {
            $jsApiList .= '"' . $jsApi . '",';
        }

        return [$wechatConfig, $jsApiList];
    }

    /**
     * 根据附件，返回可直接查看的url，这个应该放在公共方法里
     * @param $media
     * @return array|string
     */
    protected function getFullMediaUrl($media)
    {
        if (!$media) return '';
        $preg = "/http[s]?:\/\/[\w.]+[\w\/]*[\w.]*\??[\w=&\+\%]*/is";
        $mediaUrl = media($media, 'act');
        if (!preg_match($preg, $mediaUrl)) {
            $mediaUrl = $this->request->domain() . $mediaUrl;
        }
        return $mediaUrl;
    }

    /**
     * 判断当前是否是微信端
     * @return bool
     */
    protected function isWechat()
    {
        if (strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger') !== false) {
            return true;
        }
        return false;
    }

    /**
     * 判断是否为手机端
     * @return bool
     */
    protected function isMobile()
    {
        // 如果有HTTP_X_WAP_PROFILE则一定是移动设备
        if (isset ($_SERVER['HTTP_X_WAP_PROFILE'])) {
            return TRUE;
        }
        // 如果via信息含有wap则一定是移动设备,部分服务商会屏蔽该信息
        if (isset ($_SERVER['HTTP_VIA'])) {
            if (stristr($_SERVER['HTTP_VIA'], "wap")) return true;
//            return stristr($_SERVER['HTTP_VIA'], "wap") ? TRUE : FALSE;// 找不到为flase,否则为TRUE
        }
        // 判断手机发送的客户端标志,兼容性有待提高
        if (isset ($_SERVER['HTTP_USER_AGENT'])) {
            $clientkeywords = array(
                'mobile',
                'nokia',
                'sony',
                'ericsson',
                'mot',
                'samsung',
                'htc',
                'sgh',
                'lg',
                'sharp',
                'sie-',
                'philips',
                'panasonic',
                'alcatel',
                'lenovo',
                'iphone',
                'ipod',
                'blackberry',
                'meizu',
                'android',
                'netfront',
                'symbian',
                'ucweb',
                'windowsce',
                'palm',
                'operamini',
                'operamobi',
                'openwave',
                'nexusone',
                'cldc',
                'midp',
                'wap'
            );
            // 从HTTP_USER_AGENT中查找手机浏览器的关键字
            if (preg_match("/(" . implode('|', $clientkeywords) . ")/i", strtolower($_SERVER['HTTP_USER_AGENT']))) {
                return TRUE;
            }
        }
        if (isset ($_SERVER['HTTP_ACCEPT'])) { // 协议法，因为有可能不准确，放到最后判断
            // 如果只支持wml并且不支持html那一定是移动设备
            // 如果支持wml和html但是wml在html之前则是移动设备
            if ((strpos($_SERVER['HTTP_ACCEPT'], 'vnd.wap.wml') !== FALSE) && (strpos($_SERVER['HTTP_ACCEPT'], 'text/html') === FALSE || (strpos($_SERVER['HTTP_ACCEPT'], 'vnd.wap.wml') < strpos($_SERVER['HTTP_ACCEPT'], 'text/html')))) {
                return TRUE;
            }
        }
        return FALSE;
    }
}